'' PropCOMM VT100_VGA V2.0b Original By Jeff Ledger

{{
Info            : VT100 Terminal for AVR CP/M
Autor           : Joe G.
Version         : 1
Subversion      : 00
Funktion        : Keyboard Driver DE, VGA-1024x768 Driver, V24 Driver, VT100 Terminal
Komponenten     : VGA_1024
                  Keyb-DE
                  FullDuplexSerial

Log             

25.03.2013      Start
                V24 Buffer 256 Byte
                Key-DE
26.03.2013      ESC Code Terminal Reset
10.04.2013      new codes
13.04.2013      BS
14.04.2013      ESC [ Y;X H
26.04.2013      arrow Keys, cursor (not) visible, colormanagement
25.05.2013      Bell, CP/M Reset
26.05.2013      all VT100 colors, hot key Alt+i (toggle inverse)


following links maybe very valuable:

http://de.wikipedia.org/wiki/Steuerzeichen
http://de.wikipedia.org/wiki/Ascii
http://www.vt100.net/docs/vt100-ug/chapter3.html#S3.3.3
http://www.vt100.net/emu/dec_ansi_parser
http://ascii-table.com/ansi-escape-sequences-vt-100.php
 
}}

CON
   'clock configuration
   _clkmode  = xtal1 + pll16x
   _xinfreq  = 5_000_000        '5 MHz Quarz AVR CP/M

   'pin configuration 
   key_data         = 26        'Keyboard Data Pin
   key_clk          = 27        'Keyboard CLK Pin                 
   video            = 16        'Video output Pin
   serXmit          = 30        'Serial Transmit
   serRecv          = 31        'Serial Receive
   buzzer           = 1         'Buzzer/Beeper output Pin
   debugvideo       = 12        'TV out Pin (debugging)
       
   'seriell port
   baud             = 38400    'Default AVR CP/M is 115200

   ' buzzer
   use_spk_engine   = TRUE


  black             = $00 
  white             = $FC
  lred              = $40
  red               = $80
  dred              = $C0
  yellow            = $A0
  dyellow           = $F0
  green             = $20
  lgreen            = $10
  dgreen            = $30   
  blue              = $08
  lblue             = $04
  dblue             = $0C
  magenta           = $88
  cyan              = $28
  
  CR                = $0D
  LF                = $0A
  TAB               = $09
  BKS               = $08

  bspKey            = $C8
  delKey            = $BA
  ArrowTop          = $C2
  ArrowDown         = $C3
  ArrowLeft         = $C0
  ArrowRight        = $C1
                                                          
  escape            = $CB
  backspace         = 8
  fReturn           = 13
  fLinefeed         = 10
  fEof              = -1

  def_sign_color    = white
  def_back_color    = dblue

Var
  Byte  sign_color 
  Byte  back_color
  Byte  inverse
  Word  color
  Byte  index
  Byte  param[8]
  
OBJ
  text   : "VGA_1024"
  key    : "keyb-DE"                   ' new Keyboard Driver (deutsch)
  ser    : "FullDuplexSerial"          ' Full Duplex Serial Controller
  debug  : "aigeneric_driver"

PUB start | i, a, b, TimeCnt, c                 ' c nur fr Debugzwecke

  debug.start( debugvideo)
  debug.str( string("DEBUG READY", 13, 13))  

  key.start( key_data, key_clk)                         'start new keyboard driver
  text.start( video, buzzer, use_spk_engine)            'start terminal screen
  ser.start( serRecv, serXmit, %0000, baud)             'start RS232 to AVR CP/M

  sign_color := def_sign_color                          'default green
  back_color := def_back_color                          'default black
  inverse    := 0                                       'char not inverse 
  color      := (0 + back_color) << 8 + sign_color
  
  titlescreen                                           'print startmessage
 
  repeat
    terminal                                            'start VT100 Terminal Programm

 
PRI terminal | local, remote, vt100, loop, var1, col, row

   vt100 := 0
   repeat
      local := key.key
            
      if local > 0

         if local > 608 and local < 635                 'CRTL code
            local:=local-608                            'redefine as CTRL+char

         if local == 208                                'if F1
            local := 0
            text.out($D0)                               'reset inverse character                                            
            titlescreen                                 'Startmessage

         if local == 219                                'if F12
            local := 0
            text.CP_M_Reset                             'CP/M Reset                                            

         if local == 1129                               'Alt + i
            local := 0
            if inverse == 0
               inverse := 1                             'set flag
               text.out($CF)                            'set invers
               text.beep                                'beep
            else
               inverse := 0                             'rest flag
               text.out($D0)                            'normal
               text.beep                                'dobble beep
               text.beep 

         case local
          bspKey     : ser.tx($08)                      'send BS

          delKey     : ser.tx($7F)                      'send DEL

          ArrowTop   : ser.tx($1B)                      'send ESC [ A
                       ser.tx($5B)                        
                       ser.tx($41)

          ArrowDown  : ser.tx($1B)                      'send ESC [ B
                       ser.tx($5B)                        
                       ser.tx($42)                     

          ArrowRight : ser.tx($1B)                      'send ESC [ C
                       ser.tx($5B)                        
                       ser.tx($43)      

          ArrowLeft  : ser.tx($1B)                      'send ESC [ D
                       ser.tx($5B)                        
                       ser.tx($44)  
                                                                                
          OTHER      : ser.tx(local)                    'send code


      '## Deal with remote data if present ##                  
      remote := ser.rxcheck 'receive sign
      if remote == -1
        next 

      'debug.debug( remote)    ' Display everthing on the TV terminal

      case vt100
        0 :
          case remote
'            15 :                     ' ^O SI (Shift in)
            
            7  :                     ' ^H BEL
              text.beep
              
            27 :                     ' VT100 ESC code is being sent
              vt100 := 1             ' set vt100 mode

            other :  
              text.out( remote)


        1 :

           ' exit vt mode if upper case letter detected
           if remote => "<" and remote =< "Z"
             vt100 := 0

           ' exit vt mode if lower case letter detected
           if remote => "a" and remote =< "z"
             vt100 := 0

           case remote
             $18 :       ' CAN
               vt100 := 0

             $1B :       ' ESC
               vt100 := 1

             $1a :       ' SUB
               vt100 := 0

             "7" :                   ' DECSC -- Save Cursor (DEC Private)
                text.out( $D1)       ' save cursor

             "8" :                   ' DECRC -- Restore Cursor (DEC Private)
                text.out( $D2)       ' restore cursor   
                             
             "c" :  
               text.out( $D0)        ' reset inverse character                         
               text.cls              ' clear screen
               titlescreen           ' Startmessage
               
             "D" :                   ' IND -- Index
                 text.out( $C3)

             "E" :                   ' New line 
                 text.out( $0d)
                 
'            "H" :                   ' HTS -- Horizontal Tabulation Set

             "M" :                   ' RI -- Reverse Index 
                 text.out( $C2)
'
             "<" :                   ' Enter ANSI Mode
                                     ' ANSI Standards X3.64-1977 and X3.41-1974   

             "[" :
               repeat index from 0 to 7  ' clear buffer
                 param[ index] := 0
               index   := 0
               vt100   := 2           ' Control Sequence Introducer (CSI)
               
             other :
               debug.str( string("m1 not supported: "))
               debug.hex( remote, 2)
               debug.out( 13)

        2 :
           ' debug.debug( remote)     ' Display esc codes on the TV terminal

           ' exit vt mode if upper case letter detected
           if remote => "@" and remote =< "Z"
             vt100 := 0

           ' exit vt mode if lower case letter detected
           if remote => "a" and remote =< "z"
             vt100 := 0

           case remote

             $18 :       ' CAN
               vt100 := 0

             $1B :       ' ESC
               vt100 := 1

             $1a :       ' SUB
               vt100 := 0

             ' decode number to param
             ' old value * 10 + new value
             "0".."9" : 
                 param[ index] := param[ index] * 10 + ( remote - "0")

             ";" :  ' next parameter
                 index += 1

             "?" :  ' parameter is private

             "D" :                    ' CUB -- Cursor Backward
               if param[ index] == 0
                 param[ index] := 1
               repeat
                 param[ index]--
                 text.out( $C0)
               while param[ index] > 0


             "B" :                    ' CUD -- Cursor Down
               if param[ index] == 0
                 param[ index] := 1
               repeat
                 param[ index]--
                 text.out( $C3)
               while param[ index] > 0

             "C" :                    ' CUF -- Cursor Forward
               if param[ index] == 0
                 param[ index] := 1
               repeat
                 param[ index]--
                 text.out( $C1)
               while param[ index] > 0

             "A" :                    ' CUU -- Cursor Up
               if param[ index] == 0
                 param[ index] := 1
               repeat
                 param[ index]--
                 text.out( $C2)
               while param[ index] > 0

             "H", "f" :               ' CUP -- Cursor Position
                                      ' HVP -- Horizontal and Vertical Position
                if index < 1
                  index := 1
                if param[ index] == 0
                  param[ index] := 1
                if param[ index - 1] == 0
                  param[ index - 1] := 1
                row := param[ index - 1]
                col := param[ index] 
                text.cursloc( col-1, row-1)


             "J" :                    ' ED -- Erase In Display; 0, 1 not supported
               case param[ index]
                 0 : text.out( $D3) 
                 1 : text.out( $D4) 
                 2 : text.out( $D5) 

             "K" :                    ' EL -- Erase In Line; 1, 2 not supported
               if param[ index] == 0
                 text.out( $CA)
               else
                      debug.str( string( "EL mode not supported: "))
                      debug.hex( param[ index], 2)
                      debug.out( 13)
                 
             "h" :                     ' SM -- Set Mode
               repeat while index > 0
                 case param[ index]
                   25 :  text.out( $CD) ' cursor visible
                   other :                 
                      debug.str( string( "SM mode not supported: "))
                      debug.hex( param[ index], 2)
                      debug.out( 13)
                 index--
                 
             "l" :                     ' RM -- Reset Mode
               repeat while index > 0
                 case param[ index]
                   25 : text.out( $CE) ' cursor unvisible
                   other :
                      debug.str( string( "RM mode not supported: "))
                      debug.hex( param[ index], 2)
                      debug.out( 13)
                 index--

             "m" :                     ' SGR -- Select Graphic Rendition
               repeat while index > 0
                 case param[ index]
                    0 : text.out( $D0)  ' Attributes off
'                   1 :                 ' bold
'                   4 :                 ' underscore
'                   5 :                 ' blink
                    7 : text.out( $CF)  ' inverse

                   30 : sign_color := black
                   31 : sign_color := red
                   32 : sign_color := green
                   33 : sign_color := yellow
                   34 : sign_color := blue
                   35 : sign_color := magenta
                   36 : sign_color := cyan
                   37 : sign_color := white
                   39 : sign_color := def_sign_color

                   40 : back_color := black
                   41 : back_color := red
                   42 : back_color := green
                   43 : back_color := yellow
                   44 : back_color := blue
                   45 : back_color := magenta
                   46 : back_color := cyan
                   47 : back_color := white
                   49 : back_color := def_back_color    

                   other :
                      debug.str( string("SGR not supported: "))
                      debug.hex( param[ index], 2)
                      debug.out( 13)
                                   
                 color := (0 + back_color) << 8 + sign_color
                 text.color(color)   
                 ' next attribute    
                 index--
                  
             other :
               debug.str( string("m2 not supported: "))
               debug.hex( remote, 2)
               debug.out( 13)

            

 
'
'        ' ESC[L                  :function = insert line   
'         if remote == 76 and param[2] == 91                                         'look for [L
'            text.out($CB)
'            vt100:=0  
'
'        ' ESC[M                  :function = delete line   
'         if remote == 77 and param[2] == 91                                         'look for [M
'            text.out($CC)
'            vt100:=0  
'                      
'

 




PUB titlescreen

'  text.beep
  text.cls
  text.color( color)
  text.str( string( $0D,"AVR CP/M VT100 Terminal V1.0",$0D,$0A))
  text.dec( baud)
  text.str( string( ",8,N,1",$0D,$0A))
  text.str( string( "Terminal Ready",$0D,$0A))


{{

+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
}}  